pro xhelp,file,out,group=group

;+
; 
; NAME: 
;	XHELP
;
; PURPOSE:
;	Display the idl doc from a file name.
;
; CATEGORY:
;	Widgets.
;
; CALLING SEQUENCE:
;	XHELP, Filename [, text]
;
; INPUTS:
;     Filename:	A scalar string that contains the filename of the file
;		to display.  The filename can include a path to that file.
;
; OPTIONAL OUTPUTS:
;    Text: a named variable where to return the documentation text.
;
; KEYWORD PARAMETERS:
;	GROUP:	The widget ID of the group leader of the widget.  If this 
;		keyword is specified, the death of the group leader results in
;		the death of XHELP.
;
; OUTPUTS:
;	No explicit outputs.  A file viewing widget is created.
;
; SIDE EFFECTS:
;	Triggers the XMANAGER if it is not already in use. 
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Same as dl_unix, but sending the output to xdisplayfile1.
;	It calls $IDL_DIR/bin/doc_library shell script from the IDL
;	distribution.
;
; COMMON FILES: 
;	xhelp, docdir
;		(docdir is a string with a directory path, i.e.  $XOP_HOME/doc )
;		When docdir is defined, xhelp looks for the file Filename.doc
;		in the docdir directory. If it is found, then it displays
;		it, otherwise it looks for the standard procedure.
;		This is introduced  to be able to use the Help botton in
;		xop applications after the embedding, when the source will not 
;		be available.
;
; MODIFICATION HISTORY:
;	Written By Manuel Sanchez del Rio (srio@esrf.fr), August 1996
;	96-10-10 Bug corrected: it didn't consider the last directory in
;	         !path. Translate always name to lower case.
;	97-01-13 srio@esrf.fr introduces the common block and its
;		related action.
;	97-01-17 srio@esrf.fr ports to Windows95.
;	97-10-27 srio@esrf.fr sdep() used.
;-
;
on_error,2
common xhelp, docdir

separator = sdep(/ds)
if sdep() EQ 'WINDOWS' and strlen(file) GT 8 and $
  sdep(/vs) EQ '4' then file=strmid(file,0,6)+'~1'

if keyword_set(docdir) then if docdir ne '' then begin
  ffile = docdir+separator+file+'.doc'
  if checkfile(ffile) then begin
     text = read_textfile(ffile)
     xdisplayfile1,ffile,title='xhelp: '+ffile,group=group
     return
  endif
endif 

myfile = strlowcase(file)+'.pro'         ;make it always lower case

case sdep() of
'UNIX': begin
  n = strparse(!path,':',list)
  fulllist =  './'+myfile
  for i=0,n do fulllist = fulllist+' '+list(i)+'/'+myfile
  command = !dir+'/bin/doc_library cat 0 '+fulllist
  ;message,/info,'Executing: '+command
  spawn,command,out
  xdisplayfile1,text=out,title='xhelp: '+file,group=group
  end
'WINDOWS': begin
  n = strparse(!path,';',list)
  list = ['.',list]+'\'+myfile
  n=n+1
  ;openw, outunit, 'xhelp.txt',/GET_LUN
  ;printf, outunit,'xhelp search on '+systime()
  text = ''
  ifound = 0
  for i=0,n do begin
    if checkfile(list(i)) EQ 1 then begin
      ifound = ifound+1
      ;printf, outunit
      ;printf, outunit, '----- Documentation for ',list(i)
      ;printf, outunit
	text = [text,' ', '----- Documentation for '+list(i),' ']
      openr, unit, list(i), /GET_LUN
      line = ""
      outflag = 0
      readf, unit, line
      while not eof(unit) and strpos(line, ";-") ne 0 do begin
        ;if outflag then printf, outunit, strmid(line, 1, 132)
        if outflag then text = [text,strmid(line, 1, 132)]
        ;
        ; output lines after line which starts with ";+"
        ;
        if strpos(line, ";+") eq 0 then outflag = 1
        readf, unit, line
      endwhile
      free_lun,unit
      endif
  endfor
  ;free_lun,outunit
  if ifound EQ 0 then $
    itmp = widget_message(/INFO,['Entry '+myfile+' not found in path: ',$
	'Current directory + '+!path]) else  xdisplayfile1,text=text, $
	GROUP=group
  end
  else: 
endcase
end

